//////////////////////////////////////////////////////////////////////
//
// File:     OntShell.java
// Project:  MIE457F Semantic Web Reasoning System
// Author:   Scott Sanner, University of Toronto (ssanner@cs.toronto.edu)
// Date:     9/1/2003
// Requires: comshell package
//
// Description:
//
//   A command line interface to the logic.kb.fol package.
//
//////////////////////////////////////////////////////////////////////

// Package definition
package estore.ui.shell;

// Packages to import
import comshell.*;
import estore.dataman.*;
import estore.shared.*;
import java.io.*;
import java.util.*;

/**
 * Text shell interface for ontology inference system
 *
 * @version   1.0
 * @author    Scott Sanner
 * @language  Java (JDK 1.3)
 **/
public class TestInterface
{
    /** Static constants **/
    public final static int    MAX_INPUT          = 4096;
    public final static String DEFAULT_PREFS_FILE = "prefs.txt";
    public final static int    INDENT             = 3;

    /** Encapsulated Objects **/
    public CommandInterface _ci;
    public InputStream      _is;
    public PrintStream      _os;
    public EStoreInterface  _es;
    public long             _lStartTime;

    /** Class-defined commands **/
    public int QUIT;
    public int COMMENT;
    public int TIMER;
    public int ADD_ITEM;
    public int REMOVE_ITEM;
    public int GET_INV_ITEMS; 
    public int GET_SC_ITEMS; 
    public int GET_ITEM_INFO; 
    public int CHECKOUT;      
    
    /** Initializes the shell interface and invokes it
     **/
    public static void main(String args[]) 
    {
	try {

	    InputStream in;
	    if (args.length >= 1)
		in = new FileInputStream(args[0]);
	    else
		in  = System.in;
		
	    TestInterface shell = new TestInterface(in, System.out);
	    shell.run();

	} catch (FileNotFoundException e) 
	    {
		System.out.println("File IO Error while reading " + args[0] + ", exiting...");
		System.exit(1);
	    }
	
    } 
    
    /** Empty constructor - uses System input/output stream and default
     *  preferences file.
     **/
    public TestInterface()
    {
    	this(System.in, System.out, DEFAULT_PREFS_FILE);
    }

    /** Constructor - uses default preferences file.
     *  @param is InputStream to read input from
     *  @param os OutputStream to write output to
     **/
    public TestInterface(InputStream is, PrintStream os)
    {
    	this(is, os, DEFAULT_PREFS_FILE);
    }

    /** Constructor
     *  @param is InputStream to read input from
     *  @param os OutputStream to write output to
     *  @param prefs_file File to import default environmental variable
     *                    bindings/preferences from
     **/
    public TestInterface(InputStream is, PrintStream os, String prefs_file)
    {
		_is     = is;
		_os     = os;
		_ci    = new CommandInterface(_is, _os);
	
		// Initialize a set of commands
		TIMER         = _ci.command.addCommand("timer",
						       " {start|stop}                  - start or stop timer");
		QUIT          = _ci.command.addCommand("quit", 
						       "                                - quit application");
		COMMENT       = _ci.command.addCommand("//", 
						       "                                  - comment\n");
		ADD_ITEM      = _ci.command.addCommand("add", 
						       " <user> <item> <quantity>        - add items to shopping cart");
		REMOVE_ITEM   = _ci.command.addCommand("remove",
						       " <user> <item> <quantity>     - remove items from shopping cart");
		GET_ITEM_INFO = _ci.command.addCommand("info",
	       " <item>                         - get info on a specific item");
		GET_INV_ITEMS = _ci.command.addCommand("inv",
						       "                                 - list all items in inventory");
		GET_SC_ITEMS  = _ci.command.addCommand("sc",
						       " <user>                           - list all items in user's shopping cart");
		CHECKOUT      = _ci.command.addCommand("checkout",
						       " <user> <email> <payment>   - complete checkout for user's shopping cart");
		
		// Initialize environmental variable bindings from preferences file
		_ci.initEnvVarsFromFile(prefs_file);
	
		// Initialize time and other vars
		_lStartTime = 0L;
	
		// Initialize the domain
		_es = new EStore();
		_os.println("\nEStore DB successfully accessed.");
    }
    
    /** Main command line handler
     **/
    public void run()
    {
	      
		while (_ci.command.type != QUIT) {
	
		    try {
			_ci.getCommand();
		    } catch (IOException e) {
			_os.println("IO Error: " + e);
			while (true);
		    }
			
		    /***********************************************************
		     * Command: Quit
		     ***********************************************************/
		    if (_ci.command.type == QUIT) {
		    	_os.println("\nClosing down EStore interface.\n");		
		    	System.exit(0);
		    }
	
		    /***********************************************************
		     * Command: Handle timer commands
		     ***********************************************************/
		    else if (_ci.command.type == TIMER) {
	
				if (_ci.command.numParams() != 1) {
				    _os.println("\nNeed to specify either 'start' or 'stop'.");
				} else {
				    String com = _ci.command.getParam(0);
				    if (com.equalsIgnoreCase("start")) {
					_lStartTime = System.currentTimeMillis();
				    } else if (com.equalsIgnoreCase("stop")) {
					_os.println("\nElapsed time: " + (System.currentTimeMillis() - 
									  _lStartTime) + " ms");
				    } else {
					_os.println("\nUnrecognized timer command");
				    }
				}
		    }
			
		    /***********************************************************
		     * Command: Comment '//'
		     ***********************************************************/
		    if (_ci.command.type == COMMENT) {
		    	; // Do nothing
		    }
			
		    /***********************************************************
		     * Command: Add items to shopping cart
		     ***********************************************************/
		    else if (_ci.command.type == ADD_ITEM) {
	
		    	String user = null, item = null;
		    	int quantity = 0;
				if (_ci.command.numParams() == 3) {
				    user = _ci.command.getParam(0);
				    item = _ci.command.getParam(1);
				    quantity = new Integer(_ci.command.getParam(2)).intValue();
				} else {
					_os.println("3 parameters required for this command.");
					continue;
				}
		
				// Add item and print results
				if (_es.addItem(user, item, quantity) == true) {
					_os.println("\nAdd item successful.");
				} else {
					_os.println("\nAdd item could not be processed.");
				}
		    }
	
		    /***********************************************************
		     * Command: Remove items from shopping cart
		     ***********************************************************/
		    else if (_ci.command.type == REMOVE_ITEM) {
	
		    	String user = null, item = null;
		    	int quantity = 0;
				if (_ci.command.numParams() == 3) {
				    user = _ci.command.getParam(0);
				    item = _ci.command.getParam(1);
				    quantity = new Integer(_ci.command.getParam(2)).intValue();
				} else {
					_os.println("3 parameters required for this command.");
					continue;
				}
		
				
				// Add item and print results
				if (_es.removeItem(user, item, quantity) == true) {
					_os.println("\nRemove item successful.");
				} else {
					_os.println("\nRemove item could not be processed.");
				}
		    }
	
		    /***********************************************************
		     * Command: Get info for item
		     ***********************************************************/
		    else if (_ci.command.type == GET_ITEM_INFO) {
		    	
		    	String item = null;
				if (_ci.command.numParams() == 1) {
				    item = _ci.command.getParam(0);
				} else {
					_os.println("1 parameter required for this command.");
					continue;
				}
		
				// Print results
				Item i = _es.getItemInfo(item);
				if (i != null) {
					_os.println("\nItem info: " + i);
				} else {
					_os.println("\nItem info could not be processed.");
				}
		    }
	
		    /***********************************************************
		     * Command: Get all items in inventory
		     ***********************************************************/
		    else if (_ci.command.type == GET_INV_ITEMS) {
	
				if (_ci.command.numParams() >= 1) {
					_os.println("No parameters required for this command.");
					continue;
				}
		
				// Print results
				Item[] a = _es.listInventoryItems();
				if (a != null) {
					_os.println("\nInventory:\n----------");
					for (int i = 0; i < a.length; i++) {
						_os.println(a[i]);
					}
				} else {
					_os.println("\nInventory info could not be processed.");
				}
			}
	
		    /***********************************************************
		     * Command: Get all items in user's shopping cart
		     ***********************************************************/
		    else if (_ci.command.type == GET_SC_ITEMS) {
		    	
		    	String user = null;
				if (_ci.command.numParams() == 1) {
				    user = _ci.command.getParam(0);
				} else {
					_os.println("1 parameter required for this command.");
					continue;
				}
		
				// Print results
				Item[] a = _es.listSCItems(user);
				if (a != null) {
					_os.println("\nShopping cart for '" + user + 
							"':\n-----------------------------");
					for (int i = 0; i < a.length; i++) {
						_os.println(a[i]);
					}
				} else {
					_os.println("\nInventory info could not be processed.");
				}
		    }
	
		    /***********************************************************
		     * Command: Checkout shopping cart for user
		     ***********************************************************/
		    else if (_ci.command.type == CHECKOUT) {
		    	
		    	String user = null, email = null, payment = null;
				if (_ci.command.numParams() == 3) {
				    user    = _ci.command.getParam(0);
				    email   = _ci.command.getParam(1);
				    payment = _ci.command.getParam(2);
				} else {
					_os.println("3 parameters required for this command.");
					continue;
				}
		
				// Print results
				if (_es.checkout(user, email, payment)) {
					_os.println("\nCheckout successful.");
				} else {
					_os.println("\nCheckout could not be processed.");
				}
		    }
			
		}
    }

}







