/**********************************************************************
 *  Author: Alexey Tugonov
 *  Modified by: Scott Sanner
 *  
 *  Note: Because there are two types of Document objects, 
 *  org.dom4j.Document (dom4j) and org.w3c.dom.Document (DOM) which
 *  are incompatible, I have chosen to use the fully qualified class
 *  names to avoid name clashes.    
 **********************************************************************/

package estore.ui.webservice;

import java.io.*;

import estore.dataman.EStore;
import java.rmi.RMISecurityManager;
import java.util.Iterator;

import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPFactory;
import javax.xml.soap.SOAPException;
import javax.xml.soap.MessageFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.dom4j.Attribute;
import org.dom4j.Element;
import org.dom4j.DocumentHelper;
import org.dom4j.DocumentException;
import org.xml.sax.SAXException;

/** Implements the MIE354 EStore Servlet
 */
public class EStoreWebService extends WebServiceServlet {
    
	/** As usual, we need to set a security policy for our application **/
	static {
        if (System.getSecurityManager() == null) {
        	System.setProperty("java.security.policy", "mie354.policy");
            System.setSecurityManager(new RMISecurityManager());
    		System.out.println("Set security policy...");
        }
    }
	
	// Log file for messages
	public static final String LOGFILE = "estorews.log";

	// It would be a good idea to ensure that the XMLEStoreInterface
	// methods are sychronized since multiple Servlet threads can
	// access it at the same time.  But for now, this has not been done.
	public XMLEStoreInterface _xmlEStoreInterface = null;
	public SOAPFactory        _soapFactory  = null;
	public MessageFactory     _messageFactory  = null;
	public DocumentBuilder    _documentBuilder = null;
	
	// Constructor
	public EStoreWebService() {
		
		// Create a new XMLEStoreInterface which will handle all of the
		// the XML argument processing.
		_xmlEStoreInterface = new XMLEStoreInterface(new EStore());
		
		// Instantiate the following members here since we'll reuse them.
		// These are required for translation from the dom4j and DOM
		// Document models.
		try {
			_soapFactory = SOAPFactory.newInstance();
			_messageFactory = MessageFactory.newInstance();
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			_documentBuilder = dbf.newDocumentBuilder();
		} catch (Exception e) {
			System.out.println("EStoreWebService: Fatal error: " + e);
			System.exit(1);
		} 
		
	}
	
	// This is the primary method for processing SOAPMessages and producing
	// SOAPMessages as a response.  (Logging is mainly for debugging.)
	protected SOAPMessage onMessage(SOAPMessage message_in) throws SOAPException {
	
		logSOAPMessage("SOAP IN", message_in);

		// First we need to convert from a SOAPMessage to a dom4j Document
		// that is compatible with XMLEStoreInterface.
		org.dom4j.Document doc_in = SOAP2Document(message_in);
		logDocument("DOC IN", doc_in);
		
		// Now we invoke the XMLEStoreInterface and get the response.
		org.dom4j.Document doc_response = _xmlEStoreInterface.doCommand(doc_in);
		logDocument("DOC RESPONSE", doc_response);
		
		// Now we convert from a dom4j Document back to a SOAPMessage.
		SOAPMessage message_response = Document2SOAP(doc_response);
		logSOAPMessage("SOAP RESPONSE", message_response);
		
		return message_response;
	}
	
	// Helper Method: Converts a SOAPMessage to a dom4j Document.
	// Inefficient because it converts to a String and then requires reparsing.
	private org.dom4j.Document SOAP2Document(SOAPMessage m) throws SOAPException {
		
		try {
			// Retrieve the SOAPBody, its first Element is an 'input' element
			// and we want the first child element of that element.
			SOAPBody b    = m.getSOAPPart().getEnvelope().getBody();
			SOAPElement be = (SOAPElement)b.getChildElements().next();
			SOAPElement ce = (SOAPElement)be.getChildElements().next();
			
			// Convert this element to a String and parse it as a dom4j Document
			return DocumentHelper.parseText(ce.toString());
			
		} catch (DocumentException de) {
			System.out.println("ERROR: SOAP2Document: " + de);
			return null;
		} 
	}
	
	// Helper Method: Recursively converts a dom4j Document to a SOAPMessage
	// This is a reasonably efficient approach and should also be used for 
	// SOAP2Document above.
	private SOAPMessage Document2SOAP(org.dom4j.Document d) throws SOAPException {
		
		// Make a new SOAPMessage and retrieve the body
		SOAPMessage m = _messageFactory.createMessage();
		SOAPEnvelope env = m.getSOAPPart().getEnvelope();
		env.getHeader().detachNode(); // We don't need a header
		SOAPBody b = env.getBody();
		
		// Recursively convert a dom4j Document to a DOM SOAPElement and
		// add it to the SOAPMessage body.
		SOAPBodyElement be = b.addBodyElement(
				_soapFactory.createName(d.getRootElement().getQualifiedName()));
		Node2SOAPElement(d.getRootElement(), be);
		return m; // i.e., the original SOAPMessage whose body was modified
	}
	
	// Helper Method: Recursively converts a dom4j *Node* to a DOM *SOAPElement*.
	private void Node2SOAPElement(Element e, SOAPElement se) throws SOAPException {
		
		// Add the node text
		se.addTextNode(e.getText());
		
		// Add in all attributes
		for (Iterator j = e.attributeIterator(); j.hasNext(); ) {
			Attribute a = (Attribute)j.next();
    		se.addAttribute(_soapFactory.createName(a.getQualifiedName()), 
    					    a.getValue());
		}    

		// Recursively build and add in elements
		for (Iterator j = e.elementIterator(); j.hasNext(); ) {
			Element c = (Element)j.next();
			SOAPElement ce = _soapFactory.createElement(c.getQualifiedName());
			se.addChildElement(ce);
			Node2SOAPElement(c, ce); 
		}    
	}
	
	// Helper Method: Logs SOAPMessages to a file.
	private void logSOAPMessage(String prefix, SOAPMessage message) {
		try {
			
			// Log this message to a local file 
			PrintStream os = new PrintStream(new FileOutputStream(LOGFILE, true));
			os.print(prefix + ": ");
			message.writeTo(os);
			os.println("\n");
			os.close();
			
		} catch (Exception ex) {
			System.out.println("Error logging SOAPMessage: " + ex);
		}
	}
	
	// Helper Method: Logs Documents to a file.
	private void logDocument(String prefix, org.dom4j.Document message) {
		try {
			
			// Log this message to a local file 
			PrintStream os = new PrintStream(new FileOutputStream(LOGFILE, true));
			os.print(prefix + ": " + message.asXML());
			os.println("\n");
			os.close();
			
		} catch (Exception ex) {
			System.out.println("Error logging SOAPMessage: " + ex);
		}
	}

}
